"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var reduce_css_calc_1 = __importDefault(require("reduce-css-calc"));
var css_1 = __importDefault(require("css"));
var parse_1 = require("./parse");
/**
 * Checks whether a node parsed by @reworkcss/css is a css rule
 *
 * @param {css.Node} node
 * @returns Whether the node can be a css.Rule
 */
function isRule(node) {
    // could still be Page AtRule 
    return node && node.type === 'rule';
}
/**
 * Checks whether a node parsed by @reworkcss/css is a css declaration
 *
 * @param {css.Node} node
 * @returns Whether the node can be a css.Declaration
 */
function isDeclaration(node) {
    return node && node.type === 'declaration';
}
/**
 * Checks whether a declaration parsed by @reworkcss/css is a css variable
 *
 * @param {css.Declaration} declaration
 * @returns Whether the declaration is a variable
 */
function isVariable(declaration) {
    var _a;
    return (_a = declaration.property) === null || _a === void 0 ? void 0 : _a.startsWith('--');
}
/**
 * Iterates over a stylesheet's nodes to find all variables within the provided selector's scope
 *
 * @param {css.Stylesheet} stylesheet
 * @param {string} [selector] Css selector to look for variables in. Has to be exact
 * @returns A map of variables to their values
 */
function getVariablesFromStylesheet(stylesheet, selector) {
    var e_1, _a, e_2, _b;
    var res = {};
    try {
        for (var _c = __values(stylesheet.stylesheet.rules), _d = _c.next(); !_d.done; _d = _c.next()) {
            var node = _d.value;
            if (isRule(node) && (!selector || node.selectors && node.selectors.length === 1 && node.selectors.includes(selector))) {
                try {
                    for (var _e = (e_2 = void 0, __values((node.declarations || []))), _f = _e.next(); !_f.done; _f = _e.next()) {
                        var declaration = _f.value;
                        if (isDeclaration(declaration) &&
                            declaration.property &&
                            isVariable(declaration)) {
                            var property = declaration.property, value = declaration.value;
                            if (property && value)
                                res[property] = value;
                        }
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return res;
}
/**
 * Parses strings as css stylesheets to create a reference for their absolute/resolved string values
 *
 * @param {string[]} content List of strings, in order, to parse for variables
 * @param {string} [selector=':root'] A selector to scope the search for variables
 * @returns {{resolved: VariableDict, failed: string[]}} Resolved variables with absolute values + a list of the variables that failed to be resolved.
 */
function resolveCssVariables(content, selector) {
    if (selector === void 0) { selector = ':root'; }
    var rawVariables = content.reduce(function (previous, contents) { return (__assign(__assign({}, previous), getVariablesFromStylesheet(css_1.default.parse(contents), selector))); }, {});
    var resolved = {};
    var failed = {};
    var parsedVariables = {};
    for (var variable in rawVariables) {
        parsedVariables[variable] = parse_1.parseString(rawVariables[variable]);
        if (parsedVariables[variable].length === 1 && parsedVariables[variable][0].type === 'literal') {
            resolved[variable] = parsedVariables[variable][0].value;
        }
    }
    for (var variable in parsedVariables) {
        var res = parse_1.resolveFromLookups(parsedVariables[variable], resolved, parsedVariables, failed);
        if (res) {
            resolved[variable] = res;
        }
        else {
            failed[variable] = true;
        }
    }
    for (var variable in resolved) {
        // we do this here for the side-effect definitions from resolveFromLookups
        // which we should definitely refactor out in the future
        resolved[variable] = reduce_css_calc_1.default(resolved[variable]);
    }
    return {
        raw: rawVariables,
        resolved: resolved,
        failed: Object.keys(failed)
    };
}
exports.default = resolveCssVariables;
// console.log(resolveCssVariables([
//     `
//       :root {
//           --bop: black;
//           --boop: var(--bop) o;
//           --bash: throw me a var(--basher, var(--bang, var(--nope, nope)));
//           --bang: var(--goop, loop);
//           --geek: var(--nonexistent);
//           --calculated: calc(var(--one) + calc(var(--two) * 2));
//           --one: 1px;
//           --two: 1rem;
//       }
//     `
// ]));
console.log(resolveCssVariables([
    "\n  :root {\n\t  --theme-color: var(--light, white);\n\t}\n  "
]));
