"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveFromLookups = exports.parseString = void 0;
var regexes = {
    literal: /^.+?(?=var\(--.+?\)|$)/,
    variable: /^var\(--.+?(?<!\\)\)/,
    remainingBracket: /^\s*\)/,
    varStartGlobal: /var\(--/g,
    varStart: /var\(--/,
    variableName: /(?<=var\()(--.+?)(?=(,|\)))/,
};
function parseVariable(readVariable) {
    var _a, _b, _c;
    var res = {
        type: 'variable',
        rawValue: readVariable,
    };
    var variable = (_a = readVariable.match(regexes.variableName)) === null || _a === void 0 ? void 0 : _a[0].trim();
    var fallback = (_c = (_b = readVariable.match(new RegExp("(?<=^var\\(\\s*" + variable + "\\s*,\\s*)(.+?)(?=\\)$)"))) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.trim();
    res.variableName = variable;
    if (fallback !== undefined && fallback !== null) {
        if (regexes.varStart.test(fallback)) {
            res.fallback = parseVariable(fallback);
        }
        else {
            res.fallback = {
                value: fallback,
                type: 'literal'
            };
        }
    }
    else {
        res.fallback = null;
    }
    return res;
}
function parseString(value) {
    var _a, _b;
    var res = [];
    var unparsed = value;
    var currentlyMatching = 'variable';
    while (unparsed.length) {
        if (currentlyMatching === 'variable') {
            var match = unparsed.match(regexes.variable);
            if (match) {
                var matchedValue = match[0];
                var varsWithinMatchedValue = ((_a = matchedValue.match(regexes.varStartGlobal)) === null || _a === void 0 ? void 0 : _a.length);
                if (varsWithinMatchedValue > 1) {
                    // - 1 because we should already have one closing bracket
                    for (var i = 0; i < varsWithinMatchedValue - 1; i++) {
                        var temp = unparsed.slice(matchedValue.length);
                        var remainingBracket = (_b = temp.match(regexes.remainingBracket)) === null || _b === void 0 ? void 0 : _b[0];
                        // not handling missing remaining bracket.
                        // not having a remaining bracket means invalid css
                        if (!remainingBracket) {
                            throw new Error("Resolving css variables - parsed string \"" + matchedValue + "\" is missing a closing bracket");
                        }
                        matchedValue += remainingBracket;
                    }
                }
                res.push(parseVariable(matchedValue));
                unparsed = unparsed.slice(matchedValue.length);
            }
            currentlyMatching = 'literal';
        }
        else if (currentlyMatching === 'literal') {
            var match = unparsed.match(regexes.literal);
            if (match) {
                var matchedValue = match[0];
                res.push({
                    value: matchedValue,
                    type: 'literal'
                });
                unparsed = unparsed.slice(matchedValue.length);
            }
            currentlyMatching = 'variable';
        }
    }
    return res;
}
exports.parseString = parseString;
function resolveFromLookups(parsed, lookup, remainingVariables, failed) {
    var e_1, _a;
    var _resolve = function (o) {
        if (lookup[o.variableName]) {
            return lookup[o.variableName];
        }
        else {
            if (remainingVariables[o.variableName]) {
                var resolved = resolveFromLookups(remainingVariables[o.variableName], lookup, remainingVariables, failed);
                if (resolved) {
                    lookup[o.variableName] = resolved;
                    return resolved;
                }
                else {
                    failed[o.variableName] = true;
                }
            }
            else {
                failed[o.variableName] = true;
            }
            if (o.fallback) {
                if (o.fallback.type === 'variable')
                    return _resolve(o.fallback);
                else
                    return o.fallback.value;
            }
        }
    };
    var res = '';
    try {
        for (var parsed_1 = __values(parsed), parsed_1_1 = parsed_1.next(); !parsed_1_1.done; parsed_1_1 = parsed_1.next()) {
            var o = parsed_1_1.value;
            if (o.type === 'literal') {
                res += o.value;
            }
            else if (o.type === 'variable') {
                var resolved = _resolve(o);
                if (resolved === undefined || res === null) {
                    return undefined;
                }
                else {
                    res += resolved;
                }
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (parsed_1_1 && !parsed_1_1.done && (_a = parsed_1.return)) _a.call(parsed_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return res;
}
exports.resolveFromLookups = resolveFromLookups;
