"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ava_1 = __importDefault(require("ava"));
var index_js_1 = __importDefault(require("./index.js"));
ava_1.default('Returns variables for provided scope only', function (t) {
    var variables = index_js_1.default(["\n\t:root {\n\t  --dark: black;\n\t}\n\n\t.not-root {\n\t  --light: white;\n\t}\n\t"], '.not-root');
    t.deepEqual(variables.resolved, { '--light': 'white' });
});
ava_1.default('Returns variables for :root if scope not provided', function (t) {
    var variables = index_js_1.default(["\n\t:root {\n\t  --dark: black;\n\t}\n\n\t.not-root {\n\t  --light: white;\n\t}\n\t"], ':root');
    t.deepEqual(variables.resolved, { '--dark': 'black' });
});
ava_1.default('Returns values for variables thta reference variables', function (t) {
    var variables = index_js_1.default(["\n\t:root {\n\t  --dark: black;\n\t}\n\n\t:root {\n\t  --theme-color: var(--dark);\n\t}\n\t"], ':root');
    t.deepEqual(variables.resolved, { '--dark': 'black', '--theme-color': 'black' });
});
ava_1.default("Returns failed, an array with variables it couldn\'t resolve", function (t) {
    var variables = index_js_1.default(["\n\t:root {\n\t  --dark: black;\n\t}\n\n\t:root {\n\t  --theme-color: var(--light);\n\t}\n\t"], ':root');
    t.deepEqual(variables.failed.length, 2);
    t.true(variables.failed.includes('--light'));
    t.true(variables.failed.includes('--theme-color'));
});
ava_1.default("It replaces a single variable with its fallback correctly", function (t) {
    var variables = index_js_1.default(["\n\t:root {\n\t  --theme-color: var(--light, white);\n\t}\n\t"], ':root');
    t.deepEqual(variables.resolved['--theme-color'], 'white');
});
ava_1.default("It replaces a nested variable with the resolved value when resolvable", function (t) {
    var variables = index_js_1.default(["\n\t:root {\n\t  --theme-color: var(--light, var(--whitish, #fffffe));\n\t  --whitish: snow;\n\n\t  --theme-background: var(--dark, var(--darkish));\n\t  --darkish: var(--black, #000);\n\t}\n\t"], ':root');
    t.deepEqual(variables.resolved['--theme-color'], 'snow');
    t.deepEqual(variables.resolved['--theme-background'], '#000');
});
ava_1.default("It replaces a nested variable with its fallback when unresolvable", function (t) {
    var variables = index_js_1.default(["\n\t:root {\n\t  --theme-color: var(--light, var(--whitish, #fffffe));\n\t}\n\t"], ':root');
    t.deepEqual(variables.resolved['--theme-color'], '#fffffe');
});
